@@ -23,7 +23,6 @@
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <dbus/dbus-glib.h>
 #include <gio/gio.h>
 
 #include <bluetooth-applet.h>
@@ -33,8 +32,6 @@
 #include <bluetooth-killswitch.h>
 #include <bluetooth-agent.h>
 
-#include <marshal.h>
-
 static gpointer
 bluetooth_simple_device_copy (gpointer boxed)
 {
@@ -75,7 +72,7 @@ struct _BluetoothApplet
 	gulong signal_row_added;
 	gulong signal_row_changed;
 	gulong signal_row_deleted;
-	DBusGProxy* default_adapter;
+	char* default_adapter;
 	BluetoothAgent* agent;
 	GHashTable* pending_requests;
 
@@ -147,11 +144,11 @@ void bluetooth_applet_browse_address (BluetoothApplet *applet,
 {
 	g_return_if_fail (BLUETOOTH_IS_APPLET (applet));
 
-	bluetooth_applet_browse_address (applet,
-					 address,
-					 timestamp,
-					 callback,
-					 user_data);
+	bluetooth_browse_address (G_OBJECT (applet),
+				  address,
+				  timestamp,
+				  callback,
+				  user_data);
 }
 
 /**
@@ -183,20 +180,22 @@ bluetooth_applet_agent_reply_pincode (BluetoothApplet *self,
 				      const char      *request_key,
 				      const char      *pincode)
 {
-	DBusGMethodInvocation* context;
+	GDBusMethodInvocation* invocation;
 
 	g_return_if_fail (BLUETOOTH_IS_APPLET (self));
 	g_return_if_fail (request_key != NULL);
 
-	context = g_hash_table_lookup (self->pending_requests, request_key);
+	invocation = g_hash_table_lookup (self->pending_requests, request_key);
 
 	if (pincode != NULL) {
-		dbus_g_method_return (context, pincode);
+		g_dbus_method_invocation_return_value (invocation,
+						       g_variant_new ("(s)", pincode));
 	} else {
 		GError *error;
 		error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
 				     "Pairing request rejected");
-		dbus_g_method_return_error (context, error);
+		g_dbus_method_invocation_return_gerror (invocation, error);
+		g_error_free (error);
 	}
 
 	g_hash_table_remove (self->pending_requests, request_key);
@@ -214,20 +213,22 @@ bluetooth_applet_agent_reply_passkey (BluetoothApplet *self,
 				      const char      *request_key,
 				      int              passkey)
 {
-	DBusGMethodInvocation* context;
+	GDBusMethodInvocation* invocation;
 
 	g_return_if_fail (BLUETOOTH_IS_APPLET (self));
 	g_return_if_fail (request_key != NULL);
 
-	context = g_hash_table_lookup (self->pending_requests, request_key);
+	invocation = g_hash_table_lookup (self->pending_requests, request_key);
 
 	if (passkey != -1) {
-		dbus_g_method_return (context, passkey);
+		g_dbus_method_invocation_return_value (invocation,
+						       g_variant_new ("(u)", passkey));
 	} else {
 		GError *error;
 		error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
 				     "Pairing request rejected");
-		dbus_g_method_return_error (context, error);
+		g_dbus_method_invocation_return_gerror (invocation, error);
+		g_error_free (error);
 	}
 
 	g_hash_table_remove (self->pending_requests, request_key);
@@ -245,20 +246,21 @@ bluetooth_applet_agent_reply_confirm (BluetoothApplet *self,
 				      const char      *request_key,
 				      gboolean         confirm)
 {
-	DBusGMethodInvocation* context;
+	GDBusMethodInvocation* invocation;
 
 	g_return_if_fail (BLUETOOTH_IS_APPLET (self));
 	g_return_if_fail (request_key != NULL);
 
-	context = g_hash_table_lookup (self->pending_requests, request_key);
+	invocation = g_hash_table_lookup (self->pending_requests, request_key);
 
 	if (confirm) {
-		dbus_g_method_return (context);
+		g_dbus_method_invocation_return_value (invocation, NULL);
 	} else {
 		GError *error;
 		error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
 				     "Confirmation request rejected");
-		dbus_g_method_return_error (context, error);
+		g_dbus_method_invocation_return_gerror (invocation, error);
+		g_error_free (error);
 	}
 
 	g_hash_table_remove (self->pending_requests, request_key);
@@ -278,74 +280,67 @@ bluetooth_applet_agent_reply_auth (BluetoothApplet *self,
 				   gboolean         auth,
 				   gboolean         trusted)
 {
-	DBusGMethodInvocation* context;
+	GDBusMethodInvocation* invocation;
 
 	g_return_if_fail (BLUETOOTH_IS_APPLET (self));
 	g_return_if_fail (request_key != NULL);
 
-	context = g_hash_table_lookup (self->pending_requests, request_key);
+	invocation = g_hash_table_lookup (self->pending_requests, request_key);
 
 	if (auth) {
 		if (trusted)
 			bluetooth_client_set_trusted (self->client, request_key, TRUE);
 
-		dbus_g_method_return (context);
+		g_dbus_method_invocation_return_value (invocation, NULL);
 	} else {
 		GError *error;
 		error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
 				     "Confirmation request rejected");
-		dbus_g_method_return_error (context, error);
+		g_dbus_method_invocation_return_gerror (invocation, error);
+		g_error_free (error);
 	}
 
 	g_hash_table_remove (self->pending_requests, request_key);
 }
 
-#ifndef DBUS_TYPE_G_DICTIONARY
-#define DBUS_TYPE_G_DICTIONARY \
-	(dbus_g_type_get_map("GHashTable", G_TYPE_STRING, G_TYPE_VALUE))
-#endif
-
 static char *
-device_get_name (DBusGProxy *proxy, char **long_name)
+device_get_name (GDBusProxy *proxy, char **long_name)
 {
-	GHashTable *hash;
-	GValue *value;
+	GVariant *value;
+	GVariant *result;
 	char *alias, *address;
 
 	g_return_val_if_fail (long_name != NULL, NULL);
 
-	if (dbus_g_proxy_call (proxy, "GetProperties",  NULL,
-			       G_TYPE_INVALID,
-			       DBUS_TYPE_G_DICTIONARY, &hash,
-			       G_TYPE_INVALID) == FALSE) {
+	result = g_dbus_proxy_call_sync (proxy, "GetProperties",  NULL,
+					 G_DBUS_CALL_FLAGS_NONE, -1, NULL, NULL);
+	if (result == NULL)
 		return NULL;
-	}
 
-	value = g_hash_table_lookup (hash, "Address");
+	value = g_variant_lookup_value (result, "Address", G_VARIANT_TYPE_STRING);
 	if (value == NULL) {
-		g_hash_table_destroy (hash);
+		g_variant_unref (result);
 		return NULL;
 	}
-	address = g_value_dup_string (value);
+	address = g_strdup (g_variant_get_string (value, NULL));
 
-	value = g_hash_table_lookup (hash, "Name");
-	alias = value ? g_value_dup_string (value) : address;
+	value = g_variant_lookup_value (result, "Name", G_VARIANT_TYPE_STRING);
+	alias = value ? g_strdup (g_variant_get_string (value, NULL)) : g_strdup (address);
 
-	g_hash_table_destroy (hash);
+	g_variant_unref (result);
 
 	if (value)
 		*long_name = g_strdup_printf ("'%s' (%s)", alias, address);
 	else
 		*long_name = g_strdup_printf ("'%s'", address);
 
-	if (alias != address)
-		g_free (address);
+	g_free (address);
 	return alias;
 }
 
 static gboolean
-pincode_request (DBusGMethodInvocation *context,
-		 DBusGProxy            *device,
+pincode_request (GDBusMethodInvocation *invocation,
+		 GDBusProxy            *device,
 		 gpointer               user_data)
 {
 	BluetoothApplet* self = BLUETOOTH_APPLET (user_data);
@@ -354,8 +349,8 @@ pincode_request (DBusGMethodInvocation *context,
 	const char *path;
 
 	name = device_get_name (device, &long_name);
-	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
+	path = g_dbus_proxy_get_object_path (device);
+	g_hash_table_insert (self->pending_requests, g_strdup (path), invocation);
 
 	g_signal_emit (self, signals[SIGNAL_PINCODE_REQUEST], 0, path, name, long_name, FALSE);
 
@@ -366,8 +361,8 @@ pincode_request (DBusGMethodInvocation *context,
 }
 
 static gboolean
-passkey_request (DBusGMethodInvocation *context,
-		 DBusGProxy            *device,
+passkey_request (GDBusMethodInvocation *invocation,
+		 GDBusProxy            *device,
 		 gpointer               user_data)
 {
 	BluetoothApplet* self = BLUETOOTH_APPLET (user_data);
@@ -376,8 +371,8 @@ passkey_request (DBusGMethodInvocation *context,
 	const char *path;
 
 	name = device_get_name (device, &long_name);
-	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
+	path = g_dbus_proxy_get_object_path (device);
+	g_hash_table_insert (self->pending_requests, g_strdup (path), invocation);
 
 	g_signal_emit (self, signals[SIGNAL_PINCODE_REQUEST], 0, path, name, long_name, TRUE);
 
@@ -388,8 +383,8 @@ passkey_request (DBusGMethodInvocation *context,
 }
 
 static gboolean
-confirm_request (DBusGMethodInvocation *context,
-		 DBusGProxy *device,
+confirm_request (GDBusMethodInvocation *invocation,
+		 GDBusProxy *device,
 		 guint pin,
 		 gpointer user_data)
 {
@@ -399,8 +394,8 @@ confirm_request (DBusGMethodInvocation *context,
 	const char *path;
 
 	name = device_get_name (device, &long_name);
-	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
+	path = g_dbus_proxy_get_object_path (device);
+	g_hash_table_insert (self->pending_requests, g_strdup (path), invocation);
 
 	g_signal_emit (self, signals[SIGNAL_CONFIRM_REQUEST], 0, path, name, long_name, pin);
 
@@ -411,8 +406,8 @@ confirm_request (DBusGMethodInvocation *context,
 }
 
 static gboolean
-authorize_request (DBusGMethodInvocation *context,
-		   DBusGProxy *device,
+authorize_request (GDBusMethodInvocation *invocation,
+		   GDBusProxy *device,
 		   const char *uuid,
 		   gpointer user_data)
 {
@@ -422,8 +417,8 @@ authorize_request (DBusGMethodInvocation *context,
 	const char *path;
 
 	name = device_get_name (device, &long_name);
-	path = dbus_g_proxy_get_path (device);
-	g_hash_table_insert (self->pending_requests, g_strdup (path), context);
+	path = g_dbus_proxy_get_object_path (device);
+	g_hash_table_insert (self->pending_requests, g_strdup (path), invocation);
 
 	g_signal_emit (self, signals[SIGNAL_AUTHORIZE_REQUEST], 0, path, name, long_name, uuid);
 
@@ -436,17 +431,18 @@ authorize_request (DBusGMethodInvocation *context,
 static void
 cancel_request_single (gpointer key, gpointer value, gpointer user_data)
 {
-	DBusGMethodInvocation* request_context = value;
-	GError* result;
+	GDBusMethodInvocation* request_invocation = value;
+	GError* error;
 
 	if (value) {
-		result = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT, "Agent callback cancelled");
-		dbus_g_method_return_error (request_context, result);
+		error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT, "Agent callback cancelled");
+		g_dbus_method_invocation_return_gerror (request_invocation, error);
+		g_error_free (error);
 	}
 }
 
 static gboolean
-cancel_request(DBusGMethodInvocation *context,
+cancel_request(GDBusMethodInvocation *invocation,
                gpointer user_data)
 {
 	BluetoothApplet* self = BLUETOOTH_APPLET (user_data);
@@ -485,9 +481,11 @@ default_adapter_changed (GObject    *client,
 {
 	BluetoothApplet* self = BLUETOOTH_APPLET (data);
 
-	if (self->default_adapter)
-		g_object_unref (self->default_adapter);
-	self->default_adapter = bluetooth_client_get_default_adapter (self->client);
+	if (self->default_adapter) {
+		g_free (self->default_adapter);
+		self->default_adapter = NULL;
+	}
+	g_object_get (G_OBJECT (self->client), "default-adapter", &self->default_adapter, NULL);
 
 	if (self->device_model) {
 		g_signal_handler_disconnect (self->device_model, self->signal_row_added);
@@ -495,10 +493,7 @@ default_adapter_changed (GObject    *client,
 		g_signal_handler_disconnect (self->device_model, self->signal_row_deleted);
 		g_object_unref (self->device_model);
 	}
-	if (self->default_adapter)
-		self->device_model = bluetooth_client_get_device_model (self->client, self->default_adapter);
-	else
-		self->device_model = NULL;
+	self->device_model = bluetooth_client_get_device_model (self->client);
 
 	if (self->device_model) {
 		self->signal_row_added = g_signal_connect (self->device_model, "row-inserted",
@@ -522,7 +517,7 @@ default_adapter_changed (GObject    *client,
 		bluetooth_agent_set_confirm_func (self->agent, confirm_request, self);
 		bluetooth_agent_set_cancel_func (self->agent, cancel_request, self);
 
-		bluetooth_agent_register (self->agent, self->default_adapter);
+		bluetooth_agent_register (self->agent);
 	}
 
 	g_signal_emit (self, signals[SIGNAL_DEVICES_CHANGED], 0);
@@ -554,24 +549,23 @@ set_powered_foreach (GtkTreeModel *model,
                      GtkTreeIter  *iter,
                      gpointer      data)
 {
-	DBusGProxy *proxy = NULL;
-	GValue value = { 0, };
+	GDBusProxy *proxy = NULL;
 
 	gtk_tree_model_get (model, iter,
 			    BLUETOOTH_COLUMN_PROXY, &proxy, -1);
+
 	if (proxy == NULL)
 		return FALSE;
 
-	g_value_init (&value, G_TYPE_BOOLEAN);
-	g_value_set_boolean (&value, TRUE);
+	g_dbus_proxy_call (proxy,
+			   "SetProperty",
+			   g_variant_new ("sv", "Powered", g_variant_new_boolean (TRUE)),
+			   G_DBUS_CALL_FLAGS_NO_AUTO_START,
+			   -1,
+			   NULL,
+			   NULL,
+			   NULL);
 
-	dbus_g_proxy_call_no_reply (proxy, "SetProperty",
-				    G_TYPE_STRING, "Powered",
-				    G_TYPE_VALUE, &value,
-				    G_TYPE_INVALID,
-				    G_TYPE_INVALID);
-
-	g_value_unset (&value);
 	g_object_unref (proxy);
 
 	return FALSE;
@@ -617,9 +611,15 @@ typedef struct {
 } ConnectionClosure;
 
 static void
-connection_callback (BluetoothClient* client, gboolean success, gpointer data)
+
+connection_callback (GObject      *source_object,
+		     GAsyncResult *res,
+		     gpointer      user_data)
 {
-	ConnectionClosure *closure = (ConnectionClosure*) data;
+	ConnectionClosure *closure = (ConnectionClosure*) user_data;
+	gboolean success;
+
+	success = bluetooth_client_connect_service_finish (BLUETOOTH_CLIENT (source_object), res, NULL);
 
 	(*(closure->func)) (closure->self, success, closure->user_data);
 
@@ -651,7 +651,9 @@ bluetooth_applet_connect_device (BluetoothApplet* applet,
 	closure->func = func;
 	closure->user_data = data;
 
-	return bluetooth_client_connect_service (applet->client, device, connection_callback, closure);
+	bluetooth_client_connect_service (applet->client, device, TRUE, NULL, connection_callback, closure);
+
+	return TRUE;
 }
 
 /**
@@ -679,7 +681,9 @@ bluetooth_applet_disconnect_device (BluetoothApplet* applet,
 	closure->func = func;
 	closure->user_data = data;
 
-	return bluetooth_client_disconnect_service (applet->client, device, connection_callback, closure);
+	bluetooth_client_connect_service (applet->client, device, FALSE, NULL, connection_callback, closure);
+
+	return TRUE;
 }
 
 /**
@@ -692,9 +696,11 @@ bluetooth_applet_disconnect_device (BluetoothApplet* applet,
 gboolean
 bluetooth_applet_get_discoverable (BluetoothApplet* self)
 {
+	gboolean ret;
 	g_return_val_if_fail (BLUETOOTH_IS_APPLET (self), FALSE);
 
-       	return bluetooth_client_get_discoverable (self->client);
+	g_object_get (G_OBJECT (self->client), "default-adapter-discoverable", &ret, NULL);
+	return ret;
 }
 
 /**
@@ -708,7 +714,7 @@ bluetooth_applet_set_discoverable (BluetoothApplet* self, gboolean disc)
 {
 	g_return_if_fail (BLUETOOTH_IS_APPLET (self));
 
-	bluetooth_client_set_discoverable (self->client, disc, 0);
+	g_object_set (G_OBJECT (self->client), "default-adapter-discoverable", disc, NULL);
 }
 
 /**
@@ -782,7 +788,7 @@ bluetooth_applet_create_device_from_iter (GtkTreeModel *model,
 {
 	BluetoothSimpleDevice *dev;
 	GHashTable *services;
-	DBusGProxy *proxy;
+	GDBusProxy *proxy;
 	char **uuids;
 
 	dev = g_new0 (BluetoothSimpleDevice, 1);
@@ -809,7 +815,7 @@ bluetooth_applet_create_device_from_iter (GtkTreeModel *model,
 	}
 
 	if (proxy != NULL) {
-		dev->device_path = g_strdup (dbus_g_proxy_get_path (proxy));
+		dev->device_path = g_strdup (g_dbus_proxy_get_object_path (proxy));
 		g_object_unref (proxy);
 	}
 
@@ -934,7 +940,7 @@ bluetooth_applet_init (BluetoothApplet *self)
 	g_signal_connect (self->killswitch_manager, "state-changed", G_CALLBACK(killswitch_state_change), self);
 
 	self->pending_requests = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-	dbus_g_error_domain_register (AGENT_ERROR, "org.bluez.Error", AGENT_ERROR_TYPE);
+	g_dbus_error_register_error (AGENT_ERROR, AGENT_ERROR_REJECT, "org.bluez.Error.Rejected");
 
 	/* Make sure all the unblocked adapters are powered,
 	 * so as to avoid seeing unpowered, but unblocked
@@ -1004,23 +1010,23 @@ bluetooth_applet_class_init (BluetoothAppletClass *klass)
 	g_object_class_install_property (gobject_class, PROP_FULL_MENU, properties[PROP_FULL_MENU]);
 
 	signals[SIGNAL_DEVICES_CHANGED] = g_signal_new ("devices-changed", G_TYPE_FROM_CLASS (gobject_class),
-							G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
+							G_SIGNAL_RUN_FIRST | G_SIGNAL_NO_RECURSE, 0, NULL, NULL, NULL,
 							G_TYPE_NONE, 0);
 
 	signals[SIGNAL_PINCODE_REQUEST] = g_signal_new ("pincode-request", G_TYPE_FROM_CLASS (gobject_class),
-							G_SIGNAL_RUN_FIRST, 0, NULL, NULL, marshal_VOID__STRING_STRING_STRING_BOOLEAN,
+							G_SIGNAL_RUN_FIRST, 0, NULL, NULL, NULL,
 							G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
 	signals[SIGNAL_CONFIRM_REQUEST] = g_signal_new ("confirm-request", G_TYPE_FROM_CLASS (gobject_class),
-							G_SIGNAL_RUN_FIRST, 0, NULL, NULL, marshal_VOID__STRING_STRING_STRING_UINT,
+							G_SIGNAL_RUN_FIRST, 0, NULL, NULL, NULL,
 							G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT);
 
 	signals[SIGNAL_AUTHORIZE_REQUEST] = g_signal_new ("auth-request", G_TYPE_FROM_CLASS (gobject_class),
-							  G_SIGNAL_RUN_FIRST, 0, NULL, NULL, marshal_VOID__STRING_STRING_STRING_STRING,
+							  G_SIGNAL_RUN_FIRST, 0, NULL, NULL, NULL,
 							  G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
 	signals[SIGNAL_CANCEL_REQUEST] = g_signal_new ("cancel-request", G_TYPE_FROM_CLASS (gobject_class),
-						       G_SIGNAL_RUN_FIRST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID,
+						       G_SIGNAL_RUN_FIRST, 0, NULL, NULL, NULL,
 						       G_TYPE_NONE, 0);
 }
 
