@@ -140,7 +140,7 @@ void browse_callback(GObject *widget, gpointer user_data)
 			     NULL);
 		g_signal_connect(selector, "selected-device-changed",
 				 G_CALLBACK(select_device_changed), dialog);
-		gtk_container_add (GTK_CONTAINER (content_area), selector);
+		gtk_box_pack_start (GTK_BOX (content_area), selector, TRUE, TRUE, 0);
 
 		address = NULL;
 		response_id = gtk_dialog_run (GTK_DIALOG (dialog));
@@ -776,7 +776,7 @@ update_device_list (BluetoothApplet *applet,
 			if (device->type == BLUETOOTH_TYPE_KEYBOARD && program_available (GNOMECC)) {
 				add_menu_item (device->bdaddr,
 					       "keyboard",
-					       _("Open Keyboard Preferences..."),
+					       _("Keyboard Settings"),
 					       uimanager,
 					       menu_merge_id,
 					       G_CALLBACK (keyboard_callback));
@@ -784,7 +784,7 @@ update_device_list (BluetoothApplet *applet,
 			if (device->type == BLUETOOTH_TYPE_MOUSE && program_available (GNOMECC)) {
 				add_menu_item (device->bdaddr,
 					       "mouse",
-					       _("Open Mouse Preferences..."),
+					       _("Mouse and Touchpad Settings"),
 					       uimanager,
 					       menu_merge_id,
 					       G_CALLBACK (mouse_callback));
@@ -794,7 +794,7 @@ update_device_list (BluetoothApplet *applet,
 			     device->type == BLUETOOTH_TYPE_OTHER_AUDIO) && program_available (GNOMECC)) {
 				add_menu_item (device->bdaddr,
 					       "sound",
-					       _("Open Sound Preferences..."),
+					       _("Sound Settings"),
 					       uimanager,
 					       menu_merge_id,
 					       G_CALLBACK (sound_callback));
