@@ -30,8 +30,6 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-#include <dbus/dbus-glib.h>
-
 #include <bluetooth-client.h>
 #include <bluetooth-client-private.h>
 #include <bluetooth-chooser.h>
@@ -66,7 +64,7 @@ static BluetoothAgent *agent;
 
 static gchar *target_address = NULL;
 static gchar *target_name = NULL;
-static gchar *target_pincode = NULL;
+static guint target_max_digits = 0;
 static guint target_type = BLUETOOTH_TYPE_ANY;
 static gboolean target_ssp = FALSE;
 static gboolean create_started = FALSE;
@@ -144,19 +142,18 @@ set_large_label (GtkLabel *label, const char *text)
 	g_free(str);
 }
 
-static void
-update_random_pincode (void)
+static gchar* 
+get_random_pincode (guint num_digits)
 {
-	target_pincode = g_strdup_printf ("%d", g_random_int_range (pow (10, PIN_NUM_DIGITS - 1),
-								    pow (10, PIN_NUM_DIGITS) - 1));
-	automatic_pincode = FALSE;
-	g_free (user_pincode);
-	user_pincode = NULL;
+	if (num_digits == 0)
+		num_digits = PIN_NUM_DIGITS;
+	return g_strdup_printf ("%d", g_random_int_range (pow (10, num_digits - 1),
+							  pow (10, num_digits)));
 }
 
 static gboolean
-pincode_callback (DBusGMethodInvocation *context,
-		  DBusGProxy *device,
+pincode_callback (GDBusMethodInvocation *invocation,
+		  GDBusProxy *device,
 		  gpointer user_data)
 {
 	target_ssp = FALSE;
@@ -164,7 +161,8 @@ pincode_callback (DBusGMethodInvocation *context,
 	/* Only show the pincode page if the pincode isn't automatic */
 	if (automatic_pincode == FALSE)
 		gtk_assistant_set_current_page (window_assistant, PAGE_SETUP);
-	dbus_g_method_return(context, pincode);
+	g_debug ("Using pincode \"%s\"", pincode);
+	g_dbus_method_invocation_return_value (invocation, g_variant_new ("(s)", pincode));
 
 	return TRUE;
 }
@@ -174,7 +172,6 @@ restart_button_clicked (GtkButton *button,
 			gpointer user_data)
 {
 	/* Clean up old state */
-	update_random_pincode ();
 	target_ssp = FALSE;
 	target_type = BLUETOOTH_TYPE_ANY;
 	display_called = FALSE;
@@ -194,7 +191,7 @@ void
 does_not_match_cb (GtkButton *button,
 		   gpointer user_data)
 {
-	DBusGMethodInvocation *context;
+	GDBusMethodInvocation *invocation;
 	GError *error = NULL;
 	char *text;
 
@@ -208,33 +205,34 @@ does_not_match_cb (GtkButton *button,
 	gtk_label_set_text(GTK_LABEL(label_failure), text);
 	g_free(text);
 
-	context = g_object_get_data (G_OBJECT (button), "context");
-	g_error_new(AGENT_ERROR, AGENT_ERROR_REJECT,
-		    "Agent callback cancelled");
-	dbus_g_method_return(context, error);
+	invocation = g_object_get_data (G_OBJECT (button), "invocation");
+	error = g_error_new (AGENT_ERROR, AGENT_ERROR_REJECT,
+			     "Agent callback cancelled");
+	g_dbus_method_invocation_return_gerror (invocation, error);
+	g_error_free (error);
 
-	g_object_set_data (G_OBJECT(does_not_match_button), "context", NULL);
-	g_object_set_data (G_OBJECT(matches_button), "context", NULL);
+	g_object_set_data (G_OBJECT(does_not_match_button), "invocation", NULL);
+	g_object_set_data (G_OBJECT(matches_button), "invocation", NULL);
 }
 
 void
 matches_cb (GtkButton *button,
 	    gpointer user_data)
 {
-	DBusGMethodInvocation *context;
+	GDBusMethodInvocation *invocation;
 
-	context = g_object_get_data (G_OBJECT (button), "context");
+	invocation = g_object_get_data (G_OBJECT (button), "invocation");
 	gtk_widget_set_sensitive (does_not_match_button, FALSE);
 	gtk_widget_set_sensitive (matches_button, FALSE);
-	dbus_g_method_return(context, "");
+	g_dbus_method_invocation_return_value (invocation, g_variant_new ("(s)", ""));
 
-	g_object_set_data (G_OBJECT(does_not_match_button), "context", NULL);
-	g_object_set_data (G_OBJECT(matches_button), "context", NULL);
+	g_object_set_data (G_OBJECT(does_not_match_button), "invocation", NULL);
+	g_object_set_data (G_OBJECT(matches_button), "invocation", NULL);
 }
 
 static gboolean
-confirm_callback (DBusGMethodInvocation *context,
-		  DBusGProxy *device,
+confirm_callback (GDBusMethodInvocation *invocation,
+		  GDBusProxy *device,
 		  guint pin,
 		  gpointer user_data)
 {
@@ -254,15 +252,15 @@ confirm_callback (DBusGMethodInvocation *context,
 	set_large_label (GTK_LABEL (label_ssp_pin), str);
 	g_free (str);
 
-	g_object_set_data (G_OBJECT(does_not_match_button), "context", context);
-	g_object_set_data (G_OBJECT(matches_button), "context", context);
+	g_object_set_data (G_OBJECT(does_not_match_button), "invocation", invocation);
+	g_object_set_data (G_OBJECT(matches_button), "invocation", invocation);
 
 	return TRUE;
 }
 
 static gboolean
-display_callback (DBusGMethodInvocation *context,
-		  DBusGProxy *device,
+display_callback (GDBusMethodInvocation *invocation,
+		  GDBusProxy *device,
 		  guint pin,
 		  guint entered,
 		  gpointer user_data)
@@ -306,13 +304,13 @@ display_callback (DBusGMethodInvocation *context,
 	g_free(done);
 	g_free(code);
 
-	dbus_g_method_return(context);
+	g_dbus_method_invocation_return_value (invocation, NULL);
 
 	return TRUE;
 }
 
 static gboolean
-cancel_callback (DBusGMethodInvocation *context,
+cancel_callback (GDBusMethodInvocation *invocation,
 		 gpointer user_data)
 {
 	gchar *text;
@@ -329,7 +327,7 @@ cancel_callback (DBusGMethodInvocation *context,
 	gtk_label_set_text(GTK_LABEL(label_failure), text);
 	g_free(text);
 
-	dbus_g_method_return(context);
+	g_dbus_method_invocation_return_value (invocation, NULL);
 
 	return TRUE;
 }
@@ -340,15 +338,18 @@ typedef struct {
 } ConnectData;
 
 static void
-connect_callback (BluetoothClient *_client,
-		  gboolean success,
-		  gpointer user_data)
+connect_callback (GObject      *source_object,
+		  GAsyncResult *res,
+		  gpointer      user_data)
 {
 	ConnectData *data = (ConnectData *) user_data;
+	gboolean success;
+
+	success = bluetooth_client_connect_service_finish (client, res, NULL);
 
 	if (success == FALSE && g_timer_elapsed (data->timer, NULL) < CONNECT_TIMEOUT) {
-		if (bluetooth_client_connect_service(client, data->path, connect_callback, data) != FALSE)
-			return;
+		bluetooth_client_connect_service (client, data->path, TRUE, NULL, connect_callback, data);
+		return;
 	}
 
 	if (success == FALSE)
@@ -383,7 +384,11 @@ create_callback (BluetoothClient *_client,
 		 */
 		text = g_strdup_printf(_("Setting up '%s' failed"), target_name);
 
+		g_warning ("Setting up '%s' failed: %s", target_name, error->message);
+
 		gtk_label_set_markup(GTK_LABEL(label_failure), text);
+		g_free (text);
+
 		return;
 	}
 
@@ -393,14 +398,8 @@ create_callback (BluetoothClient *_client,
 	data->path = g_strdup (path);
 	data->timer = g_timer_new ();
 
-	if (bluetooth_client_connect_service(client, path, connect_callback, data) != FALSE) {
-		gtk_assistant_set_current_page (window_assistant, PAGE_FINISHING);
-	} else {
-		gtk_assistant_set_current_page (window_assistant, PAGE_SUMMARY);
-		g_timer_destroy (data->timer);
-		g_free (data->path);
-		g_free (data);
-	}
+	bluetooth_client_connect_service (client, path, TRUE, NULL, connect_callback, data);
+	gtk_assistant_set_current_page (window_assistant, PAGE_FINISHING);
 }
 
 void
@@ -409,6 +408,7 @@ close_callback (GtkWidget *assistant,
 {
 	gtk_widget_destroy(assistant);
 
+	//FIXME doesn't work with a GtkApplication
 	gtk_main_quit();
 }
 
@@ -450,7 +450,6 @@ void prepare_callback (GtkWidget *assistant,
 
 	if ((page == page_setup || page == page_connecting) && (create_started == FALSE)) {
 		const char *path = AGENT_PATH;
-		char *pin_ret;
 
 		/* Set the filter on the selector, so we can use it to get more
 		 * info later, in page_summary */
@@ -459,10 +458,8 @@ void prepare_callback (GtkWidget *assistant,
 			      NULL);
 
 		/* Do we pair, or don't we? */
-		pin_ret = get_pincode_for_device (target_type, target_address, target_name, NULL);
-		if (pin_ret != NULL && g_str_equal (pin_ret, "NULL"))
+		if (g_strcmp0 (user_pincode, "NULL") == 0)
 			path = NULL;
-		g_free (pin_ret);
 
 		g_object_ref(agent);
 		bluetooth_client_create_device (client, target_address,
@@ -476,6 +473,9 @@ void prepare_callback (GtkWidget *assistant,
 		if (automatic_pincode == FALSE && target_ssp == FALSE) {
 			char *text;
 
+			g_free (pincode);
+			pincode = get_random_pincode (target_max_digits);
+
 			if (target_type == BLUETOOTH_TYPE_KEYBOARD) {
 				text = g_strdup_printf (_("Please enter the following PIN on '%s' and press “Enter” on the keyboard:"), target_name);
 			} else {
@@ -622,6 +622,7 @@ entry_custom_event (GtkWidget *entry, GdkEventKey *event)
 void
 entry_custom_changed (GtkWidget *entry)
 {
+	g_free (user_pincode);
 	user_pincode = g_strdup (gtk_entry_get_text(GTK_ENTRY(entry)));
 	gtk_dialog_set_response_sensitive (GTK_DIALOG (pin_dialog),
 					   GTK_RESPONSE_ACCEPT,
@@ -640,104 +641,18 @@ toggle_set_sensitive (GtkWidget *button,
 	if (!active)
 		gtk_dialog_set_response_sensitive (GTK_DIALOG (pin_dialog),
 						   GTK_RESPONSE_ACCEPT, TRUE);
+	else
+		entry_custom_changed (entry_custom);
 }
 
 void
 set_user_pincode (GtkWidget *button)
 {
-	GSList *list, *l;
+	if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button)))
+		return;
 
-	list = gtk_radio_button_get_group (GTK_RADIO_BUTTON (button));
-	for (l = list; l ; l = l->next) {
-		GtkEntry *entry;
-		GtkWidget *radio;
-		const char *pin;
-
-		if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
-			continue;
-
-		/* Is it radio_fixed that changed? */
-		radio = g_object_get_data (G_OBJECT (button), "button");
-		if (radio != NULL) {
-			set_user_pincode (radio);
-			return;
-		}
-
-		pin = g_object_get_data (G_OBJECT (button), "pin");
-		entry = g_object_get_data (G_OBJECT (button), "entry");
-
-		if (entry != NULL) {
-			g_free (user_pincode);
-			user_pincode = g_strdup (gtk_entry_get_text(entry));
-			gtk_dialog_set_response_sensitive (GTK_DIALOG (pin_dialog),
-							   GTK_RESPONSE_ACCEPT,
-							   gtk_entry_get_text_length (entry) >= 1);
-		} else if (pin != NULL) {
-			g_free (user_pincode);
-			if (*pin == '\0')
-				user_pincode = NULL;
-			else
-				user_pincode = g_strdup (pin);
-		}
-
-		break;
-	}
-}
-
-static char *
-get_default_adapter_address (void)
-{
-	BluetoothClient *client;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	gboolean cont;
-
-	client = bluetooth_client_new ();
-	model = bluetooth_client_get_adapter_model (client);
-	cont = gtk_tree_model_get_iter_first(model, &iter);
-
-	while (cont == TRUE) {
-		char *address;
-		gboolean is_default;
-
-		gtk_tree_model_get (model, &iter,
-				    BLUETOOTH_COLUMN_ADDRESS, &address,
-				    BLUETOOTH_COLUMN_DEFAULT, &is_default, -1);
-
-		if (is_default == TRUE) {
-			g_object_unref (model);
-			g_object_unref (client);
-			return address;
-		}
-
-		g_free (address);
-
-		cont = gtk_tree_model_iter_next (model, &iter);
-	}
-
-	g_object_unref (model);
-	g_object_unref (client);
-
-	return NULL;
-}
-
-static char *
-pin_from_adapter (const char *bdaddr)
-{
-	char **items;
-	GString *s;
-	char *ret;
-	int i;
-
-	items = g_strsplit (bdaddr, ":", -1);
-	g_assert (g_strv_length (items) == 6);
-	s = g_string_new ("$");
-	for (i = 5; i >= 0; i--)
-		g_string_append (s, items[i]);
-	g_strfreev (items);
-	ret = g_string_free (s, FALSE);
-
-	return ret;
+	g_free (user_pincode);
+	user_pincode = g_strdup (g_object_get_data (G_OBJECT (button), "pin"));
 }
 
 void
@@ -746,17 +661,17 @@ select_device_changed (BluetoothChooser *selector,
 		       gpointer user_data)
 {
 	GValue value = { 0, };
-	char *name;
 	int legacypairing;
-	BluetoothType type;
 
 	if (gtk_assistant_get_current_page (GTK_ASSISTANT (window_assistant)) != PAGE_SEARCH)
 		return;
 
 	set_page_search_complete ();
 
-	name = bluetooth_chooser_get_selected_device_name (selector);
-	type = bluetooth_chooser_get_selected_device_type (selector);
+	/* Device was deselected */
+	if (address == NULL)
+		return;
+
 	if (bluetooth_chooser_get_selected_device_info (selector, "legacypairing", &value) != FALSE) {
 		legacypairing = g_value_get_int (&value);
 		if (legacypairing == -1)
@@ -769,41 +684,19 @@ select_device_changed (BluetoothChooser *selector,
 	target_address = g_strdup (address);
 
 	g_free(target_name);
-	target_name = name;
+	target_name = bluetooth_chooser_get_selected_device_name (selector);
 
-	target_type = type;
+	target_type = bluetooth_chooser_get_selected_device_type (selector);
 	target_ssp = !legacypairing;
 	automatic_pincode = FALSE;
 
 	g_free (pincode);
 	pincode = NULL;
 
-	if (user_pincode != NULL && *user_pincode != '\0') {
-		pincode = g_strdup (user_pincode);
-		automatic_pincode = TRUE;
-	} else if (address != NULL) {
-		guint max_digits;
-
-		pincode = get_pincode_for_device(target_type, target_address, target_name, &max_digits);
-		if (pincode == NULL) {
-			/* Truncate the default pincode if the device doesn't like long
-			 * PIN codes */
-			if (max_digits != PIN_NUM_DIGITS && max_digits > 0)
-				pincode = g_strndup(target_pincode, max_digits);
-			else
-				pincode = g_strdup(target_pincode);
-		} else if (g_str_equal (pincode, "WII")) {
-			char *default_adapter;
-
-			g_free (pincode);
-			default_adapter = get_default_adapter_address ();
-			pincode = pin_from_adapter (default_adapter);
-			g_free (default_adapter);
-			automatic_pincode = TRUE;
-		} else if (target_ssp == FALSE) {
-			automatic_pincode = TRUE;
-		}
-	}
+	g_free (user_pincode);
+	user_pincode = get_pincode_for_device (target_type, target_address, target_name, &target_max_digits);
+	automatic_pincode = user_pincode != NULL;
+	gtk_entry_set_max_length (GTK_ENTRY (entry_custom), target_max_digits);
 }
 
 void
@@ -811,39 +704,33 @@ pin_option_button_clicked (GtkButton *button,
 			       gpointer data)
 {
 	GtkWidget *radio;
-	char *oldpin;
-
-	oldpin = user_pincode;
-	user_pincode = NULL;
 
 	gtk_window_set_transient_for (GTK_WINDOW (pin_dialog),
 				      GTK_WINDOW (window_assistant));
 	gtk_window_present (GTK_WINDOW (pin_dialog));
 
 	/* When reopening, try to guess where the pincode was set */
-	if (oldpin == NULL)
+	if (user_pincode == NULL)
 		radio = radio_auto;
-	else if (g_str_equal (oldpin, "0000"))
+	else if (g_str_equal (user_pincode, "0000"))
 		radio = radio_0000;
-	else if (g_str_equal (oldpin, "1111"))
+	else if (g_str_equal (user_pincode, "1111"))
 		radio = radio_1111;
-	else if (g_str_equal (oldpin, "1234"))
+	else if (g_str_equal (user_pincode, "1234"))
 		radio = radio_1234;
-	else
+	else if (g_str_equal (user_pincode, "NULL"))
+		radio = radio_none;
+	else {
 		radio = radio_custom;
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
-	if (radio == radio_custom)
-		gtk_entry_set_text (GTK_ENTRY (entry_custom), oldpin);
-
-	if (gtk_dialog_run (GTK_DIALOG (pin_dialog)) != GTK_RESPONSE_ACCEPT) {
-		g_free (user_pincode);
-		user_pincode = oldpin;
-	} else {
-		g_free (oldpin);
+		gtk_entry_set_text (GTK_ENTRY (entry_custom), user_pincode);
 	}
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
 
-	gtk_widget_hide(pin_dialog);
+	gtk_dialog_run (GTK_DIALOG (pin_dialog));
+	gtk_widget_hide (pin_dialog);
+	g_free (pincode);
+	pincode = g_strdup (user_pincode);
+	automatic_pincode = user_pincode != NULL;
 }
 
 static int
@@ -866,9 +753,6 @@ create_wizard (void)
 {
 	GtkAssistant *assistant;
 	GError *err = NULL;
-	GtkWidget *combo, *page_intro;
-	GtkTreeModel *model;
-	GtkCellRenderer *renderer;
 
 	builder = gtk_builder_new ();
 	if (gtk_builder_add_from_file (builder, "wizard.ui", NULL) == 0) {
@@ -891,24 +775,7 @@ create_wizard (void)
 	gtk_widget_hide (button_close);
 	gtk_widget_hide (button_cancel);
 
-	/* Intro page */
-	combo = gtk_combo_box_new();
-
-	model = bluetooth_client_get_adapter_model(client);
-	gtk_combo_box_set_model(GTK_COMBO_BOX(combo), model);
-	g_object_unref(model);
-
-	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
-	gtk_cell_layout_clear(GTK_CELL_LAYOUT(combo));
-
-	renderer = gtk_cell_renderer_text_new();
-	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), renderer, TRUE);
-	gtk_cell_layout_set_attributes(GTK_CELL_LAYOUT(combo), renderer,
-					"text", BLUETOOTH_COLUMN_NAME, NULL);
-
-	page_intro = W("page_intro");
-	if (gtk_tree_model_iter_n_children(model, NULL) > 1)
-		gtk_box_pack_start(GTK_BOX(page_intro), combo, FALSE, FALSE, 0);
+	/* Intro page, nothing to do */
 
 	/* Search page */
 	page_search = W("page_search");
@@ -957,11 +824,12 @@ create_wizard (void)
 	radio_custom = W("radio_custom");
 	entry_custom = W("entry_custom");
 
-	g_object_set_data (G_OBJECT (radio_auto), "pin", "");
+	g_object_set_data (G_OBJECT (radio_auto), "pin", NULL);
 	g_object_set_data (G_OBJECT (radio_0000), "pin", "0000");
 	g_object_set_data (G_OBJECT (radio_1111), "pin", "1111");
 	g_object_set_data (G_OBJECT (radio_1234), "pin", "1234");
 	g_object_set_data (G_OBJECT (radio_none), "pin", "NULL");
+	g_object_set_data (G_OBJECT (radio_custom), "pin", "");
 	g_object_set_data (G_OBJECT (radio_custom), "entry", entry_custom);
 
 	gtk_builder_connect_signals(builder, NULL);
@@ -1019,8 +887,6 @@ int main (int argc, char **argv)
 
 	gtk_window_set_default_icon_name("bluetooth");
 
-	update_random_pincode ();
-
 	client = bluetooth_client_new();
 
 	agent = bluetooth_agent_new();
